//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>

#include "rec/robotino/com/all.h"
#include "rec/core_lt/utils.h"
#include "rec/core_lt/Timer.h"
#include "rec/core_lt/StringList.h"

using namespace rec::robotino::com;

class MyCom : public Com
{
public:
	MyCom()
	{
	}

	void errorEvent( Error error, const char* errorString )
	{
		std::cerr << "Error: " << errorString << std::endl;
	}

	void connectedEvent()
	{
		std::cout << "Connected." << std::endl;
	}

	void connectionClosedEvent()
	{
		std::cout << "Connection closed." << std::endl;
	}

	void modeChangedEvent( bool isPassiveMode )
	{
		if( isPassiveMode )
		{
			std::cout << "Connected int passive mode." << std::endl;
		}
	}
};

bool _readPressures = false;

class MyBHA : public CompactBHA
{
public:
	MyBHA()
	{
	}

	virtual void readingsCallback( const std::vector<float>& pressures )
	{
		if( _readPressures )
		{
			_readPressures = false;
			for( unsigned int i=0; i<pressures.size(); ++i )
			{
				std::cout << "B" << i+1 << ": " << pressures[i] << " bar" << std::endl;
			}
			std::cout << std::endl;
		}
	}

	virtual void error( const std::string& message )
	{
		std::cerr << message << std::endl;
	}

	virtual void log( const std::string& message )
	{
		std::cerr << message << std::endl;
	}

	virtual void connected()
	{
		std::cerr << "Connected" << std::endl;
	}

	virtual void disconnected()
	{
		std::cerr << "Disconnected" << std::endl;
	}
};


//MyCom com;
MyBHA bha;

void init( const std::string& hostname )
{
	// Initialize the actors

	// Connect
	std::cout << "Connecting..." << std::endl;
	bha.setAddress( hostname.c_str() );

	bha.connectToServer();

	rec::core_lt::msleep( 2000 );
}

void drive()
{
	const float maxPressure = 1.5f;
	std::vector<float> pressures;
	pressures.resize( 8 );
	for( unsigned int i=0; i<8; ++i )
	{
		pressures[i] = 0.0f;
	}

	while( bha.isConnected() )
	{
		std::cout << "b n p - b command, n bellows number[1-8], p pressure in bar" << std::endl;
		std::cout << "c s x y - c command, s segment[1-2], x x-coord, y y-coord" << std::endl;
		std::cout << "r to read pressures" << std::endl;
		std::cout << "q to quit" << std::endl;
		std::cout << "Enter command: ";
		
		std::string command;
		std::cin >> command;

		if( "q" == command )
		{
			break;
		}
		else if( "r" == command )
		{
			_readPressures = true;
			std::cout << std::endl;
			rec::core_lt::msleep( 500 );
			continue;
		}
		else if( "b" == command )
		{
			unsigned int n;
			std::cin >> n;

			if( n > 8 || n < 1 )
			{
				std::cout << "n out of range" << std::endl;
			}
			else
			{
				float p;
				std::cin >> p;

				if( p < 0.0f )
				{
					p = 0.0f;
				}
				else if( p > maxPressure )
				{
					p = maxPressure;
				}

				std::cout << "Set bellows " << n << " to " << p << " bar." << std::endl;
				pressures[n-1] = p;
			}
		}
		else if( "c" == command )
		{
			unsigned int seg;
			std::cin >> seg;

			if( seg < 1 || seg > 2 )
			{
				std::cout << "s out of range" << std::endl;
			}
			else
			{
				float x, y;
				std::cin >> x;

				std::cin >> y;

				CompactBHASimple::xy2pressure( x, y, &pressures[0 + (seg-1)*3], &pressures[1 + (seg-1)*3], &pressures[2 + (seg-1)*3] );
				pressures[0] *= maxPressure;
				pressures[1] *= maxPressure;
				pressures[2] *= maxPressure;
			}
		}
		else
		{
			std::cout << "Unknown command" << std::endl;
		}

		for( int i=0; i<8; ++i )
		{
			std::cout << "Set bellows " << i+1 << " to " << pressures[i] << std::endl;
		}

		bha.setPressures( pressures );

		std::cout << std::endl;
	}
}

void destroy()
{
	bha.disconnectFromServer();
}

int main( int argc, char **argv )
{
	std::string hostname = "192.168.1.212:8380";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	try
	{
		init( hostname );
		drive();
		destroy();
	}
	catch( const rec::robotino::com::ComException& e )
	{
		std::cerr << "Com Error: " << e.what() << std::endl;
	}
	catch( const std::exception& e )
	{
		std::cerr << "Error: " << e.what() << std::endl;
	}
	catch( ... )
	{
		std::cerr << "Unknow Error" << std::endl;
	}

	//std::cout << "Press any key to exit..." << std::endl;
	//rec::core_lt::waitForKey();
}
