//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <iostream>
#include <map>

#include "rec/robotino/com/all.h"
#include "rec/core_lt/utils.h"
#include "rec/core_lt/Timer.h"

#include <boost/thread/mutex.hpp>

bool _run = true;

class MyManipulator : public rec::robotino::com::Manipulator
{
public:
	MyManipulator()
	{
	}

	virtual void servoInfoCallback( const int numServos )
	{
		std::cout << "Num servos: " << numServos << std::endl << std::flush;
	}

	virtual void readingsCallback( const rec::robotino::com::ManipulatorReadings& readings )
	{
		boost::mutex::scoped_lock lk( _mutex ); 
		_readings = readings;
	}

	virtual void limitsCallback( const rec::robotino::com::ManipulatorLimits& limits )
	{
		std::cout << "limitsCallback" << std::endl;
		
		boost::mutex::scoped_lock lk( _mutex ); 
		_limits = limits;
	}


	virtual void error( const std::string& message )
	{
		_run = false;
		std::cout << "Error: " << message << std::endl;
	}

	virtual void connected()
	{
		std::cout << "connected" << std::endl;
	}

	virtual void disconnected()
	{
		_run = false;
		std::cout << "disconnected" << std::endl;
	}

	boost::mutex _mutex;

	rec::robotino::com::ManipulatorLimits _limits;
	rec::robotino::com::ManipulatorReadings _readings;
};

class MyCom : public rec::robotino::com::Com
{
	public:
		MyCom()
		{
		}

		void errorEvent( Error error, const char* errorString )
		{
			std::cerr << "Error: " << errorString << std::endl;
			_run = false;
		}

		void connectedEvent()
		{
			std::cout << "Connected." << std::endl;
		}

		void connectionClosedEvent()
		{
			std::cout << "Connection closed." << std::endl;
			_run = false;
		}
};

MyManipulator manip;

void drive()
{
	while( _run && 0 == manip._limits.seq )
	{
		std::cout << "Waiting for limits" << std::endl;
		rec::core_lt::msleep( 1000 );
	}

	std::vector< float > cwLimits;
	cwLimits = manip._limits.cwAxesLimits;

	std::vector< float > ccwLimits;
	ccwLimits = manip._limits.ccwAxesLimits;

	for( unsigned int i=0; i<cwLimits.size(); ++i )
	{
		if( cwLimits[i] < 100 )
		{
			cwLimits[i] = 100;
		}
	}

	for( unsigned int i=0; i<ccwLimits.size(); ++i )
	{
		if( ccwLimits[i] > 150 )
		{
			ccwLimits[i] = 150;
		}
	}

	std::vector< float > speeds;
	speeds.resize( manip._limits.cwAxesLimits.size() );

	for( unsigned int i=0; i<speeds.size(); ++i )
	{
		speeds[i] = 10.0f;
	}

	const float speed_delta = 0.01f;

	unsigned int seq = 0;

	while( _run )
	{
		if( 0 == seq % 2 )
		{
			manip.setAxes( cwLimits, speeds );
		}
		else
		{
			manip.setAxes( ccwLimits, speeds );
		}

		for( int i=0; i<5; ++i )
		{
			if( false == _run )
			{
				return;
			}
			rec::core_lt::msleep( 1000 );
		}

		++seq;
	}
}

int main( int argc, char **argv )
{
	std::string hostname = "127.0.0.1:8080";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	MyCom com;
	manip.setComId( com.id() );

	try
	{
		com.setAddress( hostname.c_str() );
		com.connect();
		drive();
	}
	catch( const rec::robotino::com::ComException& e )
	{
		std::cout << e.what() << std::endl;
	}

	//std::cout << "Press any key to exit..." << std::endl;
	//rec::core_lt::waitForKey();
}
