//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>

#include "rec/robotino/com/all.h"

#ifdef WIN32
#include <windows.h>
// _getch
#include <conio.h>
#else
// getchar
#include <stdio.h>
// usleep
#include <unistd.h>

#include "rec/sharedmemory/sharedmemory.h"
#include "rec/iocontrol/robotstate/State.h"

#endif

void waitForKey()
{
#ifdef WIN32
	_getch();
#else
	::getchar();
#endif
}

void msleep( unsigned int ms )
{
#ifdef WIN32
	SleepEx( ms, false );
#else
	::usleep( ms * 1000 );
#endif
}

//#define VELOCITY 350.0f //mm/s
//#define ANGULARVELOCITY 5.0f //20.0f //deg/s

#define SLOW_VELOCITY 200.0f //mm/s
#define MEDIUM_VELOCITY 300.0f //mm/s
#define VELOCITY 400.0f //mm/s
#define FAST_VELOCITY 480.0f //mm/s
#define ANGULARVELOCITY 5.0f //20.0f //deg/s

using namespace rec::robotino::com;

class MyCom : public Com
{
public:
	MyCom()
	{
	}

	void errorEvent( Error error, const char* errorString )
	{
		std::cerr << "Error: " << errorString << std::endl;
	}

	void connectedEvent()
	{
		std::cout << "Connected." << std::endl;
	}

	void connectionClosedEvent()
	{
		std::cout << "Connection closed." << std::endl;
	}
};

class MyInfo : public Info
{
public:
	MyInfo()
	{
	}

	void infoReceivedEvent( const char* text )
	{
		std::cout << "Info message received ..." << std::endl;
		std::cout << text << std::endl;
		std::cout << std::endl;
	}
};

MyCom com;
Bumper bumper;
DistanceSensor sensors[9];
OmniDrive omniDrive;
MyInfo info;

//rotate vector in by a degrees and store the output in out
void rotate( const double* in, double* out, double a )
{
	double rad = 2 * M_PI / 360.0f * a;

	out[0] = cos( rad ) * in[0] - sin( rad ) * in[1];
	out[1] = sin( rad ) * in[0] + cos( rad ) * in[1];
}

void rotateInPlace( double* v, double a )
{
	double rad = 2 * M_PI / 360.0f * a;
	double tmp = v[0];

	v[0] = cos( rad ) * v[0] - sin( rad ) * v[1];
	v[1] = sin( rad ) * tmp + cos( rad ) * v[1];
}

void addScaledVector( double* srcDest, const double* uv, double scale )
{
	srcDest[ 0 ] += uv[ 0 ] * scale;
	srcDest[ 1 ] += uv[ 1 ] * scale;
}

void normalizeVector( double* v )
{
	double len = sqrt( v[0]*v[0] + v[1]*v[1] );
	v[0] /= len;
	v[1] /= len;
}

float getNextNeighbourDistance( int index )
{
	return sensors[ (index+1) % 9 ].voltage();
}

float getPrevNeighbourDistance( int index )
{
	return sensors[ (index+8) % 9 ].voltage();
}

void init( const std::string& hostname )
{
	// Initialize the actors
	info.setComId( com.id() );
	bumper.setComId( com.id() );
	for( int i = 0; i < 9; i++ )
	{
		sensors[i].setComId( com.id() );
		sensors[i].setSensorNumber( i );
	}
	omniDrive.setComId( com.id() );

	// Connect
	std::cout << "Connecting..." << std::endl;
	com.setAddress( hostname.c_str() );

	com.connect();

	std::cout << std::endl << "Connected" << std::endl;
}

void drive()
{
	const double ev[2] = {1.0f, 0.0f};

	// escape vector for distance sensor
	double escapeVector[9][2] =
	{
		{0.0f, 0.0f},
		{0.0f, 0.0f},
		{0.0f, 0.0f},
		{0.0f, 0.0f},
		{0.0f, 0.0f},
		{0.0f, 0.0f},
		{0.0f, 0.0f},
		{0.0f, 0.0f},
		{0.0f, 0.0f}
	};

	for( unsigned int i=0; i<9; i++ )
	{
		rotate( ev, escapeVector[i], 40.0f * i );
	}

	static const float ESCAPE_V = 2.2f;
	static const float WALL_LOST_V = 0.9f;
	static const float WALL_FOUND_V = 1.1f;
	static const float WALL_FOLLOW_V = 1.7f;
	static const float NEW_WALL_FOUND_V = 1.9f;
	double escape[2];
	int curWallSensor = -1;
	double dir[2] = {1.0, 0.0};
	float velocity = VELOCITY;
	float rotVelocity = ANGULARVELOCITY;

	while( com.isConnected() )
	{
		velocity = VELOCITY;
		int maxIndex = 0;
		float maxValue = 0.0f;
		unsigned int numEscape = 0;
		escape[0] = escape[1] = 0.0;
		for( unsigned int i = 0; i < 9; ++i )
		{
			float v = sensors[ i ].voltage();
			if( v > maxValue )
			{
				maxValue = v;
				maxIndex = i;
			}
			if( v > ESCAPE_V )
			{
				++numEscape;
				addScaledVector( escape, escapeVector[i], v );
			}
		}
		if( numEscape >= 2 )
		{
			// close to walls with more than one sensor, try to escape
			normalizeVector( escape );
			rotate( escape, dir, 180 );
			velocity = SLOW_VELOCITY;
			//std::cerr << "escaping..." << std::endl;
		}
		else
		{
			if( curWallSensor != -1 && sensors[ curWallSensor ].voltage() < WALL_LOST_V )
			{
				//std::cerr << "wall lost" << std::endl;
				curWallSensor = -1;
				rotateInPlace( dir, -20 ); 
				rotVelocity = -20;
			}
			if( curWallSensor == -1 )
			{
				// wall not found yet
				if( maxValue > WALL_FOUND_V )
				{
					//std::cerr << "!!!!!! wall found: " << maxIndex << std::endl;
					curWallSensor = maxIndex;
					velocity = SLOW_VELOCITY;
				}
			}
			if( curWallSensor != -1 )
			{
				float wallDist = sensors[ curWallSensor ].voltage();
				//std::cerr << "old walldist: " << wallDist << std::endl;
				// check for global new wall
				if( maxIndex != curWallSensor && maxValue > NEW_WALL_FOUND_V )
				{
					// new wall found, drive along this wall
					//std::cerr << "!!!!!! new wall found: " << maxIndex << std::endl;
					curWallSensor = maxIndex;
					wallDist = sensors[ curWallSensor ].voltage();
					velocity = SLOW_VELOCITY;
				}
				else
				{
					if( sensors[ (curWallSensor + 1)%9 ].voltage() > wallDist )
					{
						// switch walls
						curWallSensor = (curWallSensor+1)%9;
						velocity = MEDIUM_VELOCITY;
					}
					// check for new wall in direction
					for( unsigned int i = 0; i < 2; ++i )
					{
						int tmpId = (curWallSensor + 2 + i) % 9;
						if( sensors[ tmpId ].voltage() > WALL_FOUND_V )
						{
							//std::cerr << "!!!!!! wall switched in direction: " << tmpId << std::endl;
							curWallSensor = tmpId;
							wallDist = sensors[ tmpId ].voltage();
							velocity = SLOW_VELOCITY;
							break;
						}
					}
				}
				// try to keep neighbor distance sensors in balance
				float vr = getNextNeighbourDistance( curWallSensor );
				float vl = getPrevNeighbourDistance( curWallSensor );
				rotVelocity = (vr  - vl);
				float followAngle = 95;
				if( fabs( rotVelocity ) > 0.9 )
				{
					velocity = SLOW_VELOCITY;
					followAngle = rotVelocity >= 0.0 ? 140 : 80;
				}
				else if( fabs( rotVelocity ) > 0.4 )
				{
					velocity = MEDIUM_VELOCITY;
					followAngle = rotVelocity >= 0.0 ? 120 : 85;
				}
				rotVelocity *= 8 * ANGULARVELOCITY;
				//        std::cerr << "rot: " << rotVelocity << std::endl;
				// follow the wall to the left
				rotate( escapeVector[ curWallSensor ], dir, followAngle );
				// keep distance to wall steady
				float scale = WALL_FOLLOW_V - wallDist;
				if( scale > 0 )
				{
					scale *= 0.2;
				}
				addScaledVector( dir, escapeVector[ curWallSensor ], scale );
				normalizeVector( dir );
			}
			else
			{
				// no wall in sight, drive old direction
				//std::cerr << "no wall" << std::endl;
			}
		}
		if( maxValue < 0.9 )
		{
			velocity = FAST_VELOCITY;
		}
		omniDrive.setVelocity( velocity * (float)(dir[0]), velocity * (float)(dir[1]), rotVelocity );
		com.waitForUpdate();
	}
}

void destroy()
{
	com.disconnect();
}

int main( int argc, char **argv )
{
	std::string hostname = "172.26.1.1";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

#ifndef WIN32
	rec::sharedmemory::SharedMemory< rec::iocontrol::robotstate::State >* mem = new rec::sharedmemory::SharedMemory< rec::iocontrol::robotstate::State >( rec::iocontrol::robotstate::State::sharedMemoryKey );
	bool emergencyStopState;
	{
		rec::sharedmemory::Lock lk( mem );
		rec::iocontrol::robotstate::State* state = mem->getData();
		emergencyStopState = state->emergencyStop.isEnabled;
		state->emergencyStop.isEnabled = false;
	}
#endif
	try
	{
		init( hostname );
		drive();
		destroy();
	}
	catch( const rec::robotino::com::ComException& e )
	{
		std::cerr << "Com Error: " << e.what() << std::endl;
	}
	catch( const std::exception& e )
	{
		std::cerr << "Error: " << e.what() << std::endl;
	}
	catch( ... )
	{
		std::cerr << "Unknow Error" << std::endl;
	}

#ifndef WIN32
	{
		rec::sharedmemory::Lock lk( mem );
		rec::iocontrol::robotstate::State* state = mem->getData();
		state->emergencyStop.isEnabled = emergencyStopState;
	}
#endif

	//std::cout << "Press any key to exit..." << std::endl;
	//waitForKey();
}
