//  Copyright (C) 2004-2010, Robotics Equipment Corporation GmbH

#ifndef _REC_ROBOTINO_COM_MANIPULATOR_H_
#define _REC_ROBOTINO_COM_MANIPULATOR_H_

#include "rec/robotino/com/Actor.h"
#include "rec/robotino/com/ManipulatorReadings.h"
#include "rec/robotino/com/ManipulatorLimits.h"

#include <vector>
#include <string>

namespace rec
{
	namespace robotino
	{
		namespace com
		{
			class ManipulatorImpl;

			/**
			* @brief	Access to Robotino's (optional) manipulator
			*/
			class
#ifdef WIN32
#  ifdef rec_robotino_com_EXPORTS
		__declspec(dllexport)
#  endif
#  ifdef rec_robotino_com2_EXPORTS
		__declspec(dllexport)
#  endif
#  ifdef rec_robotino_com3_EXPORTS
		__declspec(dllexport)
#  endif
#else
#  ifndef SWIG
    __attribute__ ((visibility ("default")))
#  endif
#endif
			Manipulator : public Actor
			{
			public:
				Manipulator();

				virtual ~Manipulator();

				/** 
				* Sets the associated communication object.
				*
				* @param id	The id of the associated communication object.
				* @throws	RobotinoException If the given communication object doesn't provide a manipulator.
				*/
				void setComId( const ComId& id );

				/**
				Normally the manipulator's data exchange client is connected to Robotino by calling Com::connect().
				For debugging you might want to only connect to the manipulator server on Robotino.
				* @see Com::connect
				*/
				void connectToServer();

				/**
				Normally the manipulator's data exchange client is disconnected from Robotino by calling Com::disconnect().
				For debugging you might want to only disconnect from the manipulator server on Robotino.
				* @see Com::disconnect
				*/
				void disconnectFromServer();

				/**
				Normally the address of the manipulator server is given implicitly by calling Com::setAddress
				For debugging you might want to set the address explicitly.
				* @see Com::setAddress
				*/
				void setAddress( const std::string& address );

				std::string address() const;

				bool isConnected() const;

				/**
				Set position and speed set-point for one axis.
				@param axis Axis number. Axes are counted starting with 0.
				@param angle This is the position set-point in deg
				@param speed This is the speed set-point in rpm
				*/
				void setAxis( unsigned int axis, float angle, float speed );

				/**
				Set position and speed set-points for all axes.
				@param angles These are the position set-points in deg for all axes
				@param speeds These are the speed set-points in rpm for all axes
				*/
				void setAxes( const std::vector<float>& angles, const std::vector<float>& speeds );

				/**
				Enable/Disable power of a servo channel
				@param channel 0 - RX64
				               1 - RX28
							   2 - RX10
				@param enable True - enable
				              False - disable
				*/
				void setPowerEnabled( unsigned int channel, bool enable );

				/**
				Toggle the servo torque status.
				*/
				void toggleTorque();

				virtual void servoInfoCallback( const int numServos );

				/**
				Called when new manipulator data is available.
				@readings The sensor readings.
				*/
				virtual void readingsCallback( const ManipulatorReadings& readings );

				/**
				Called when axes limits are received from manipulator
				@limits Axes limits
				*/
				virtual void limitsCallback( const ManipulatorLimits& limits );

				/**
				Called when an error occured
				@message The error message
				*/
				virtual void error( const std::string& message );

				/**
				Called when connected to armd
				*/
				virtual void connected();

				/**
				Called when disconnected from armd
				*/
				virtual void disconnected();

			private:
				ManipulatorImpl* _impl;
			};
		}
	}
}
#endif
